@@ -1,5 +1,8 @@
+import json
 import os
+import pathlib
 import shutil
+from functools import partial
 from multiprocessing.pool import Pool
 
 import click
@@ -8,8 +11,10 @@ from tqdm import tqdm
 
 from rebotics_sdk.cli.common import shell, configure, roles, set_token
 from .utils import ReboticsCLIContext, pass_rebotics_context, app_dir, read_saved_role, process_role
+from ..advanced import remote_loaders
 from ..providers import DatasetProvider, ProviderHTTPClientException
-from ..utils import download_file, mkdir_p
+from ..providers.utils import get_id_list_from_ranges
+from ..utils import download_file, mkdir_p, get_filename_from_url
 
 
 @click.group()
@@ -165,6 +170,48 @@ def download_ref_backup(ctx, retailer, facenet, output):
         shutil.copyfileobj(raw, out_file)
 
 
+def task_download_and_set_training_data(item_id, target, ctx):
+    response = ctx.provider.get_detection_image_by_id(item_id)
+    # download images and download this json
+    image_url = response['image']
+    json_file = target / "{}.json".format(item_id)
+    image_file = target / "{}.{ext}".format(
+        item_id,
+        ext=get_filename_from_url(image_url).split(".")[-1]
+    )
+    try:
+        remote_loaders.download(image_url, str(image_file))
+    except:
+        click.echo("{} failed to download image from {}".format(item_id, image_url))
+    with open(str(json_file), 'w') as fio:
+        json.dump(response, fio, indent=4)
+
+
+@api.command()
+@click.argument("image_ids", type=click.STRING)
+@click.option('-t', '--target', type=click.Path(), default='.')
+@click.option('-c', '--concurrency', type=int, default=4)
+@pass_rebotics_context
+def download_images(ctx, image_ids, target, concurrency):
+    # parse image_ids as range or as a ids
+    id_list = get_id_list_from_ranges(image_ids)
+    if ctx.verbose:
+        click.echo(id_list)
+
+    if not id_list:
+        click.echo("Invalid ID list is supplied, failed to extract ids")
+
+    mkdir_p(str(target))
+
+    with Pool(min(len(id_list), concurrency)) as pool:
+        pool.map(
+            partial(task_download_and_set_training_data, target=pathlib.Path(target), ctx=ctx),
+            id_list
+        )
+    pool.join()
+    pool.close()
+
+
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
